'use strict';

function _interopDefault (ex) { return (ex && (typeof ex === 'object') && 'default' in ex) ? ex['default'] : ex; }

var videojs = _interopDefault(require('video.js'));
var elementResizeDetectorMaker = _interopDefault(require('element-resize-detector'));
var entries = _interopDefault(require('object.entries'));

var version = "1.0.1";

var _typeof = typeof Symbol === "function" && typeof Symbol.iterator === "symbol" ? function (obj) {
  return typeof obj;
} : function (obj) {
  return obj && typeof Symbol === "function" && obj.constructor === Symbol && obj !== Symbol.prototype ? "symbol" : typeof obj;
};

// Default options for the plugin.
var defaults$$1 = {
  sizes: {
    mini: 450,
    small: 600
  },
  controls: {
    currentTimeDisplay: {
      mini: false
    },
    timeDivider: {
      mini: false
    },
    durationDisplay: {
      mini: false
    },
    remainingTimeDisplay: {
      mini: false,
      small: false
    },
    captionsButton: {
      mini: false
    }
  }
};

// // Cross-compatibility for Video.js 5 and 6.
var registerPlugin = videojs.registerPlugin || videojs.plugin;
// // const dom = videojs.dom || videojs;

/**
 * Ascending sorting for breakpoints
 *
 * @param      {Array}  previousSize  Previous breakpoint to sort
 * @param      {Array}  nextSize      Next breakpoint to sort
 * @return     {Array}  Breakpoints array sorted in ascending order
 */
var ascending = function ascending(previousSize, nextSize) {
  return previousSize[1] - nextSize[1];
};

/**
 * Gets all active breakpoints that may be applied to current video player size.
 *
 * @param      {number}    playerSize  The player size.
 * @param      {Object}    sizes       Object containing all possible breakpoints.
 * @return     {Function}  The active breakpoints.
 */
var getActiveBreakpoints = function getActiveBreakpoints(playerSize, sizes) {
  if (!Object.entries) {
    entries.shim();
  }

  return Object.entries(sizes).sort(ascending).filter(function (size) {
    return size[1] >= playerSize;
  });
};

/**
 * Gets the breakpoint name for the current player's size
 *
 * @param      {Player}   player  Video.js player object.
 * @param      {Object}   sizes   Object containing all possible breakpoints.
 * @return     {string}   Breakpoint name.
 */
var getPlayerSize = function getPlayerSize(player, sizes) {
  var playerSize = player.el().clientWidth;
  var breakpoints = getActiveBreakpoints(playerSize, sizes);

  return breakpoints[0] ? breakpoints[0][0] : 'default';
};

/**
 * Controls should be visible by default so this function checks if
 * desired state was specified and returns either desired state or true.
 *
 * @param      {boolean}  setting  Desired state.
 * @return     {boolean}  Desired state or true.
 */
var trueByDefault = function trueByDefault(setting) {
  return typeof setting !== 'undefined' ? !!setting : true;
};

/**
 * Determines if given control is a native Video.js plugin by searching for it
 * in player.controlBar.
 *
 * @param      {Player}   player   Video.js player object.
 * @param      {string}   control  Name of the plugin.
 * @return     {boolean}  True if native, False otherwise.
 */
var isNative = function isNative(player, control) {
  return _typeof(player.controlBar[control]) === 'object';
};

/**
 * Uses Video.js API to hide or show Video.js plugin.
 *
 * @param      {Player}   player   Video.js player object.
 * @param      {Object}   control  The element to trigger.
 * @param      {boolean}  show     Desired state.
 * @return     {Function} Control element
 */
var setNative = function setNative(player, control, show) {
  var target = player.controlBar[control];

  return show ? target.show() : target.hide();
};

/**
 * Uses CSS to hide or show custom element. Useful for legacy plugins that
 * do not follow official plugin structure.
 *
 * @param      {Player}     player      Video.js player object.
 * @param      {string}     className   Class of the element to trigger.
 * @param      {boolean}    show        Desired state.
 * @return     {(DOMElement|undefined)} Control element
 */
var setCustom = function setCustom(player, className, show) {
  var target = player.el().querySelectorAll('.' + className);
  var hiddenClass = 'vjs-hidden';

  if (target.length === 0) {
    return undefined;
  }

  return show ? target[0].classList.remove(hiddenClass) : target[0].classList.add(hiddenClass);
};

/**
 * Applies specified settingf to the control element based on video player's size
 *
 * @param      {Player}         player   Video.js player object.
 * @param      {Object|string}  control  Video.js plugin instance or class name.
 * @param      {boolean}        setting  Indites if the control element should be visible
 *                                       on given breakpoint.
 * @return     {(DOMElement|Object)} Control element
 */
var set$$1 = function set$$1(player, control, setting) {
  var target = trueByDefault(setting);
  var native = isNative(player, control);

  return native ? setNative(player, control, target) : setCustom(player, control, target);
};

/**
 * Applies specified settings to the video player based on it's size
 *
 * @param      {Player}  player         Video.js player object.
 * @param      {Object}  arg2           Settings object
 * @param      {Object}  arg2.sizes     Breakpoints on which the controls will update.
 * @param      {Object}  arg2.controls  Specifies states of all the controls
 *                                      for different breakpoints.
 * @return     {undefined}
 */
var setup = function setup(player, _ref) {
  var sizes = _ref.sizes,
      controls = _ref.controls;

  var size = getPlayerSize(player, sizes);

  for (var control in controls) {
    var setting = controls[control][size];

    set$$1(player, control, setting);
  }
};

/**
 * Initializes new instance of resize detector
 *
 * @return     {ElementResizeDetector} Detector instance
 */
var newDetector = function newDetector() {
  return elementResizeDetectorMaker({ strategy: 'scroll' });
};

/**
 * Overwrites default breakpoints with ones specified by the developer.
 *
 * @param      {Object}  settings  Settings passed on initialisation.
 * @param      {Object}  defaultSettings  Default settings for plugin.
 * @return     {Object}  Settings to be applied.
 */
var getMediaQueries = function getMediaQueries(settings, defaultSettings) {
  return settings && settings.sizes ? settings.sizes : defaultSettings.sizes;
};

/**
 * Uses deep-merge to merge default controls settings with ones
 * specified by the developer.
 *
 * @param      {Object}  settings  Settings paased on initialisation.
 * @param      {Object}  defaultSettings  Default settings for plugin.
 * @return     {Object}  Settings to be applied.
 */
var mergeUserSettings = function mergeUserSettings(settings, defaultSettings) {
  return settings && settings.controls ? videojs.mergeOptions(defaultSettings.controls, settings.controls) : defaultSettings.controls;
};

/**
 * A video.js plugin.
 *
 * In the plugin function, the value of `this` is a video.js `Player`
 * instance. You cannot rely on the player being in a "ready" state here,
 * depending on how the plugin is invoked. This may or may not be important
 * to you; if not, remove the wait for "ready"!
 *
 * @function responsiveControls
 * @param    {Object} [userSettings={}]
 *           An object of options left to the plugin author to define.
 */
var responsiveControls = function responsiveControls(userSettings) {
  var _this = this;

  var settings = {
    sizes: getMediaQueries(userSettings, defaults$$1),
    controls: mergeUserSettings(userSettings, defaults$$1)
  };

  this.addClass('vjs-responsive-controls');
  this.resizeDetector = this.resizeDetector || newDetector();
  this.resizeDetector.listenTo(this.el(), function () {
    return setup(_this, settings);
  });
};

// Register the plugin with video.js.
registerPlugin('responsiveControls', responsiveControls);

// Include the version number.
responsiveControls.VERSION = version;

module.exports = responsiveControls;
